/**
 * 
 * Copyright (c) 2009 Emanuil Rusev - http://erusev.com
 * 
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * The software is provided "as is", without warranty of any kind,
 * express or implied, including but not limited to the warranties
 * of merchantability, fitness for a particular purpose and
 * noninfringement. In no event shall the authors or copyright
 * holders be liable for any claim, damages or other liability,
 * whether in an action of contract, tort or otherwise, arising
 * from, out of or in connection with the software or the use or
 * other dealings in the software.
 *
 */

package com.erusev.media
{
	import flash.display.Sprite;
	import flash.display.Stage;
	import flash.events.Event;
	import flash.events.EventDispatcher;
	import flash.events.IOErrorEvent;
	import flash.events.ProgressEvent;
	import flash.media.Sound;
	import flash.media.SoundChannel;
	import flash.media.SoundTransform;
	import flash.net.URLRequest;
	
	public class Sound_ extends Sprite
	{
		public function Sound_()
		{
			super();
			
			soundTransform_ = new SoundTransform();
		}
		
		private var sound:Sound;
		private var soundChannel:SoundChannel;
		private var soundTransform_:SoundTransform;
		
		////////////////////////////////////////
		// properties
		////////////////////////////////////////
		
		// url
		////////////////////////////////////////
		
		private var _url:String;
		
		public function set url(value:String):void
		{
			stop();
			close();
			playheadPosition = 0;
			
			sound = new Sound();
			sound.addEventListener(ProgressEvent.PROGRESS, sound_onProgress);
			sound.addEventListener(IOErrorEvent.IO_ERROR, sound_onIOError);
			sound.addEventListener(Event.OPEN, sound_onOpen);
			
			_url = value;
		}
		
		// volume
		////////////////////////////////////////
		
		public function get volume():Number
		{
			return soundTransform_.volume;
		}
		
		public function set volume(value:Number):void
		{
			value = Math.max(value, 0);
			value = Math.min(value, 1);
			
			soundTransform_.volume = value;
			
			if (soundChannel)
			{
				soundChannel.soundTransform = soundTransform_;
			}
			
			dispatchEvent(new Event("volumeChange"));
		}
		
		////////////////////////////////////////
		// read-only
		////////////////////////////////////////
		
		// playheadTime (read-only)
		////////////////////////////////////////
		
		private var _playheadPosition:Number;
		
		public function get playheadPosition():Number
		{
			if ( ! isNaN(_playheadPosition))
			{
				return _playheadPosition;
			}
			else if (soundChannel)
			{
				return soundChannel.position
			}
			else
			{
				return 0;
			}
		}
		
		public function set playheadPosition(value:Number):void
		{
			_playheadPosition = value;
			
			dispatchEvent(new Event("playheadUpdate"));
		}
		
		// estimatedLength (read-only)
		////////////////////////////////////////
		
		private var _estimatedDuration:Number = 0;
		
		public function get estimatedDuration():Number
		{
			return _estimatedDuration;
		}
		
		// playing (read-only)
		////////////////////////////////////////
		
		private var _playing:Boolean;
		
		public function get playing():Boolean
		{
			return _playing;
		}
		
		// portionLoaded (read-only)
		////////////////////////////////////////
		
		private var _loadedPortion:Number;
		
		public function get loadedPortion():Number
		{
			return _loadedPortion;
		}
		
		// elapsedPortion (read-only)
		////////////////////////////////////////
		
		public function get elapsedPortion():Number
		{
			// round the value to fix a bug related to soundChangel.position
			return Math.round(playheadPosition/_estimatedDuration*1000)/1000;
		}
		
		////////////////////////////////////////
		// public methods
		////////////////////////////////////////
		
		public function load():void
		{
			sound.load(new URLRequest(_url));
		}
		
		public function play():void
		{
			if ( ! sound.bytesLoaded) load();
			
			if (_playing) stop();
			
			soundChannel = sound.play(playheadPosition, -1, soundTransform_);
			soundChannel.addEventListener(Event.SOUND_COMPLETE, soundChannel_onSoundComplete);
			
			addEventListener(Event.ENTER_FRAME, onEnterFrame);
			
			_playing = true;
			
			dispatchEvent(new Event("play"));
		}
		
		public function stop():void
		{
			if ( ! soundChannel) return;
			
			soundChannel.stop();
			
			removeEventListener(Event.ENTER_FRAME, onEnterFrame);
			
			_playing = false;
			
			dispatchEvent(new Event("stop"));
		}
		
		public function close():void
		{
			try
			{
				sound.close();				
			}
			catch (e:Error)
			{
				
			}
			
			dispatchEvent(new Event("close"));
		}
		
		// event handlers
		////////////////////////////////////////
		
		private function onEnterFrame(e:Event):void
		{
			_playheadPosition = NaN;
			
			dispatchEvent(new Event("playheadUpdate"));
		}
		
		private function soundChannel_onSoundComplete(e:Event):void
		{
			stop();
			
			dispatchEvent(new Event("complete"));
		}
		
		private function sound_onProgress(e:ProgressEvent):void
		{
			_loadedPortion = e.bytesLoaded/e.bytesTotal;
			_estimatedDuration = sound.length/_loadedPortion;
			
			dispatchEvent(new Event("loadProgress"));
		}
		
		private function sound_onIOError(e:IOErrorEvent):void
		{
			
		}
		
		private function sound_onOpen(e:Event):void
		{
			dispatchEvent(e.clone());
		}
	}
}
