package mx.core
{
	import flash.display.Sprite;
	import flash.events.Event;
	
	/** Simulates Flex's UIComponent. */
	
	public class UIComponent extends Sprite
	{
		public function UIComponent()
		{
			super();
			
			addEventListener(Event.ADDED_TO_STAGE, onAddedToStage);
			addEventListener(Event.ENTER_FRAME, onEnterFrame);
		}
		
		private var specifiedWidth:Number;
		private var specifiedHeight:Number;
		
		protected var measuredWidth:Number = 0; 
		protected var measuredHeight:Number = 0;
		
		private var initialized:Boolean;
		
		private var invalidatePropertiesFlag:Boolean;
		private var invalidateSizeFlag:Boolean;
		private var invalidateDisplayListFlag:Boolean;
		
		public function get unscaledWidth():Number
		{
			if (specifiedWidth)
			{
				return specifiedWidth;
			}
			else
			{
				return measuredWidth;
			}
		}
		
		public function get unscaledHeight():Number
		{
			if (specifiedHeight)
			{
				return specifiedHeight;
			}
			else
			{
				return measuredHeight;
			}
		}
		
		override public function get width():Number
		{
			return unscaledWidth;
		}
		
		override public function set width(value:Number):void
		{
			specifiedWidth = value;
			
			invalidateDisplayList();
		}
		
		override public function get height():Number
		{
			return unscaledHeight;
		}
		
		override public function set height(value:Number):void
		{
			specifiedHeight = value;
			
			invalidateDisplayList();
		}
		
		protected function invalidateProperties():void
		{
			invalidatePropertiesFlag = true;
			addEventListener(Event.ENTER_FRAME, onEnterFrame);
		}
		
		protected function invalidateSize():void
		{
			invalidateSizeFlag = true;
			addEventListener(Event.ENTER_FRAME, onEnterFrame);
		}
		
		protected function invalidateDisplayList():void
		{
			invalidateDisplayListFlag = true;
			addEventListener(Event.ENTER_FRAME, onEnterFrame);
		}
		
		protected function createChildren():void
		{
			
		}
		
		protected function commitProperties():void
		{
			
		}
		
		protected function measure():void
	    {
	        
	    }
		
		protected function updateDisplayList(unscaledWidth:Number, unscaledHeight:Number):void
		{
			
		}
		
		private function onAddedToStage(e:Event):void
		{
			invalidateProperties();
			invalidateSize();
			invalidateDisplayList();
			
			removeEventListener(Event.ADDED_TO_STAGE, onAddedToStage);
		}
		
		private function onEnterFrame(e:Event):void
		{
			if (!initialized)
			{
				initialized = true;
				
				createChildren();
			}
			
			if (invalidatePropertiesFlag)
			{
				invalidatePropertiesFlag = false;
				commitProperties();
			}
			
			if (invalidateSizeFlag)
			{
				invalidateSizeFlag = false;
				measure();
			}
			
			if (invalidateDisplayListFlag)
			{
				invalidateDisplayListFlag = false;
				updateDisplayList(unscaledWidth, unscaledHeight);
			}
			
			removeEventListener(Event.ENTER_FRAME, onEnterFrame);
		}
	}
}